/*
 * PortMidi component of the MIDI testing module.
 *
 * See test.cpp for details.
 */

#include <iostream>
#include <portmidi.h>
#include <porttime.h>
#include <assert.h>
#undef NDEBUG // Don't let VC override the assertions in Release mode


using namespace std;

const int PM_LATENCY_HISTOGRAM_BINS = 21;
const int INPUT_BUFFER_SIZE = 100;
const int OUTPUT_BUFFER_SIZE = 0;

const int SETTLE_TIME = 5000;		// msec
const int STRAGGLE_TIME = 1000;		// msec (must be multiple of 1000)

const int MAX_NOTES_PER_ITERATION = 100; // must be set to <= 127

struct MidiData
{
   int period;				// ms per callback

   long num_iterations;
   
   long histogram[PM_LATENCY_HISTOGRAM_BINS];
   long histogram_neg[PM_LATENCY_HISTOGRAM_BINS];
   long histogram_out[PM_LATENCY_HISTOGRAM_BINS];
   long histogram_in[PM_LATENCY_HISTOGRAM_BINS];
   long sum, sumSq;
   long sum_out, sumSq_out;
   long sum_in, sumSq_in;
   long max_latency;			// worst latency observed
   long max_neg_latency;		// worst negative latency
   long max_latency_out;
   long max_latency_in;
   long out_of_range;			// num. outside of the bins 
   long out_of_range_neg;
   long out_of_range_out;
   long out_of_range_in;

   bool settled;			// so main program can easily
					// ..know when this has happened
   
   bool test_in, test_out, test_thru;	// what are we doing?
   int out_num;				// how many msgs per iteration out?
   enum { NOTES, ACTIVESENSE } out_type;// what type of messages?
   PmStream *in, *out;			// PortMidi streams
   PtTimestamp previous_callback_time;
   bool note_on;			// Is/are the note(s) on right now?
};

void initialize_midi_data(MidiData& data);

PtTimestamp Pt_Time_wrapper(void* dummy);

/**
 * The callback.
 */
void pt_callback(PtTimestamp timestamp, void *userData);

/**
 * Display/Set routines for MIDI-input/-output devices (set routines
 * ask user to choose one, display routines just list to stream).
 */
bool Pt_is_settled(MidiData * data);

#include<fstream>
void Pm_DisplayInputDevice(std::ostream &to);
int Pm_PromptForInputDevice();
void Pm_DisplayOutputDevice(std::ostream &to);
int Pm_PromptForOutputDevice();

/**
 * Debug for functions that don't take a stream parameter.
 */
void Pm_Debug(PmError error);

/**
 * Debug for functions that do take a stream parameter, which should be passed
 * here as well.
 */
void Pm_DebugStream(PmError error, PortMidiStream* stream);
